/*  ============================================================================
 *   Copyright (c) Texas Instruments Inc 2012
 *
 *   Use of this software is controlled by the terms and conditions found in the
 *   license agreement under which this software has been supplied.
 *   ===========================================================================
 */
/** ============================================================================
 example for SPI EEPROM driver on Keystone DSP. 
 Tested on AT225640B and N25Q128.
 * =============================================================================
 *  Revision History
 *  ===============
 *  Feb 20, 2012 Brighton  file created
 * =============================================================================*/

#include <stdio.h>
#include "Keystone_common.h"
#include "Keystone_SPI_init_drv.h"
#include "SPI_EEPROM_drv.h"

Uint32 uiSPI_EEPROM_page_size=32; 	/*must be power of 2*/
Uint32 uiSPI_EEPROM_sector_size=64*1024;	/*must be power of 2*/
Bool needEraseBeforeWrite= FALSE; 	

/*read "uiByteCount" data to "ucBuffer" from "data_address" of 
SPI EEPROM connected to "SPI_CS".
return number of bytes transfered*/
Uint32 SPI_EEPROM_read(Uint32 SPI_CS, Uint32 data_address, 
	Uint32 address_width, Uint32 uiByteCount, Uint8 * ucBuffer)
{
	Uint8 txBuf[4];
	Uint32 byteSuccess=0;
	
	txBuf[0]= SPI_EEPROM_READ;	/*instruction*/
	if(address_width==24)
	{
		txBuf[1]= data_address>>16; 	/*higher 8 bits of the address*/
		txBuf[2]= data_address>>8; 	/*middle 8 bits of the address*/
		txBuf[3]= data_address; 	/*lower 8 bits of the address*/
	}
	else
	{
		txBuf[1]= data_address>>8; 	/*higher 8 bits of the address*/
		txBuf[2]= data_address; 	/*lower 8 bits of the address*/
	}
	/*read data*/
	byteSuccess= KeyStone_SPI_TxRx(txBuf, 0, address_width/8+1,	
		ucBuffer, address_width/8+1, uiByteCount, 
		SPI_CS_NO_LAST_HOLD, TRUE, SPI_CS, SPI_CS);

	if(byteSuccess<address_width/8+1)
		byteSuccess= 0;
	else
		byteSuccess -= address_width/8+1;
	return byteSuccess;
}

/*write "uiByteCount" data in "ucBuffer" to "data_address" of SPI EEPROM
connected to "SPI_CS".
Please note, the data must be in SPI_EEPROM_PAGE_SIZE, that is,
((data_address&0xff)+uiByteCount)<=SPI_EEPROM_PAGE_SIZE
return number of bytes transfered.*/
Uint32 SPI_EEPROM_page_write(Uint32 SPI_CS, Uint32 data_address, 
	Uint32 address_width, Uint32 uiByteCount, Uint8 * ucBuffer)
{
	Uint8 spiBuffer[256+4];
	Uint32 byteSuccess=0;

	spiBuffer[0]= SPI_EEPROM_WREN;	/*instruction*/
	/*Write enable instruction*/
	byteSuccess= KeyStone_SPI_TxRx(spiBuffer, 0, 1,	
		NULL, 0, 0, SPI_CS_NO_HOLD, TRUE, SPI_CS, SPI_CS);

	spiBuffer[0]= SPI_EEPROM_WRITE;	/*instruction*/
	if(address_width==24)
	{
		spiBuffer[1]= data_address>>16; 	/*higher 8 bits of the address*/
		spiBuffer[2]= data_address>>8; 	/*middle 8 bits of the address*/
		spiBuffer[3]= data_address; 	/*lower 8 bits of the address*/
	}
	else
	{
		spiBuffer[1]= data_address>>8; 	/*higher 8 bits of the address*/
		spiBuffer[2]= data_address; 	/*lower 8 bits of the address*/
	}

	memcpy(spiBuffer+address_width/8+1, ucBuffer, uiByteCount);

	/*Write data*/
	byteSuccess= KeyStone_SPI_TxRx(spiBuffer, 0, address_width/8+1+uiByteCount,	
		NULL, 0, 0, SPI_CS_NO_LAST_HOLD, TRUE, SPI_CS, SPI_CS);

	if(byteSuccess<address_width/8+1)
		return 0;
	byteSuccess -= address_width/8+1;

	/*poll the status register until programming complete*/
	spiBuffer[0]= SPI_EEPROM_RDSR;	/*instruction*/
	do
	{
		KeyStone_SPI_TxRx(spiBuffer, 0, 1,	
			spiBuffer+1, 1, 1, SPI_CS_NO_LAST_HOLD, TRUE, SPI_CS, SPI_CS);
	}while(spiBuffer[1]&1);
	
	return byteSuccess;
}

/*write "uiByteCount" data in "ucBuffer" to address "data_address" 
in the SPI EEPROM connected to "SPI_CS".*/
Uint32 SPI_EEPROM_write(Uint32 SPI_CS, Uint32 data_address, 
	Uint32 address_width, Uint32 uiByteCount, Uint8 * ucBuffer)
{
	Uint32 pageByteCount, pageAddrOffset;
	Uint32 byteSuccess, tempCount= uiByteCount;

	/*SPI EEPROM write must be in SPI_EEPROM_PAGE_SIZE, write more than 
	SPI_EEPROM_PAGE_SIZE a time will wrap around and overwrite privious data*/

	/*checke the address offset in a SPI_EEPROM_PAGE_SIZE*/
	pageAddrOffset= data_address&(uiSPI_EEPROM_page_size-1);

	/*handle non-aligned data firstly*/
	if(pageAddrOffset)
	{
		pageByteCount= uiSPI_EEPROM_page_size- pageAddrOffset;
		if(pageByteCount>uiByteCount)
			pageByteCount= uiByteCount;

		byteSuccess= SPI_EEPROM_page_write(SPI_CS, data_address, 
			address_width, pageByteCount, ucBuffer);
		if(byteSuccess!= pageByteCount)
			return byteSuccess;

		data_address+= pageByteCount;
		ucBuffer+= pageByteCount;
		tempCount-= pageByteCount;
	}

	/*write the remaining data page by page*/
	while(tempCount)
	{
		if(tempCount>=uiSPI_EEPROM_page_size)
			pageByteCount= uiSPI_EEPROM_page_size;
		else
			pageByteCount= tempCount;

		byteSuccess= SPI_EEPROM_page_write(SPI_CS, data_address, 
			address_width, pageByteCount, ucBuffer);
		if(byteSuccess!= pageByteCount)
			return byteSuccess;

		data_address+= pageByteCount;
		ucBuffer+= pageByteCount;
		tempCount-= pageByteCount;
	}

	return uiByteCount;
}

/*erase the whole EEPROM*/
Uint32 SPI_EEPROM_bulk_erase(Uint32 SPI_CS)
{
	Uint8 spiBuffer[4];
	Uint32 byteSuccess=0;

	if(!needEraseBeforeWrite)
		return 0;

	spiBuffer[0]= SPI_EEPROM_WREN;	/*instruction*/
	byteSuccess= KeyStone_SPI_TxRx(spiBuffer, 0, 1,	
		NULL, 0, 0, SPI_CS_NO_HOLD, TRUE, SPI_CS, SPI_CS);
	if(1!=byteSuccess)
		return 0;

	spiBuffer[0]= SPI_EEPROM_BULK_ERASE;	/*instruction*/
	byteSuccess= KeyStone_SPI_TxRx(spiBuffer, 0, 1,	
		NULL, 0, 0, SPI_CS_NO_HOLD, TRUE, SPI_CS, SPI_CS);
	if(1!=byteSuccess)
		return 0;

	/*poll the status register until programming complete*/
	spiBuffer[0]= SPI_EEPROM_RDSR;	/*instruction*/
	do
	{
		KeyStone_SPI_TxRx(spiBuffer, 0, 1,	
			spiBuffer+1, 1, 1, SPI_CS_NO_LAST_HOLD, TRUE, SPI_CS, SPI_CS);
	}while(spiBuffer[1]&1);
	
	return byteSuccess;
}

/*erase one sector in EEPROM*/
Uint32 SPI_EEPROM_sector_erase(Uint32 SPI_CS, Uint32 sector_address, 
	Uint32 address_width)
{
	Uint8 spiBuffer[4];
	Uint32 byteSuccess=0;

	if(!needEraseBeforeWrite)
		return 0;

	spiBuffer[0]= SPI_EEPROM_WREN;	/*instruction*/
	/*Write enable instruction*/
	byteSuccess= KeyStone_SPI_TxRx(spiBuffer, 0, 1,	
		NULL, 0, 0, SPI_CS_NO_HOLD, TRUE, SPI_CS, SPI_CS);
	if(1!=byteSuccess)
		return 0;

	spiBuffer[0]= SPI_EEPROM_SECTOR_ERASE;	/*instruction*/
	if(address_width==24)
	{
		spiBuffer[1]= sector_address>>16; 	/*higher 8 bits of the address*/
		spiBuffer[2]= sector_address>>8; 	/*middle 8 bits of the address*/
		spiBuffer[3]= sector_address; 	/*lower 8 bits of the address*/
	}
	else
	{
		spiBuffer[1]= sector_address>>8; 	/*higher 8 bits of the address*/
		spiBuffer[2]= sector_address; 	/*lower 8 bits of the address*/
	}
	byteSuccess= KeyStone_SPI_TxRx(spiBuffer, 0, address_width/8+1,	
		NULL, 0, 0, SPI_CS_NO_LAST_HOLD, TRUE, SPI_CS, SPI_CS);
	if(4!=byteSuccess)
		return 0;

	/*poll the status register until programming complete*/
	spiBuffer[0]= SPI_EEPROM_RDSR;	/*instruction*/
	do
	{
		KeyStone_SPI_TxRx(spiBuffer, 0, 1,	
			spiBuffer+1, 1, 1, SPI_CS_NO_LAST_HOLD, TRUE, SPI_CS, SPI_CS);
	}while(spiBuffer[1]&1);
	
	return byteSuccess;
}

/*erase a section in EEPROM, return number of sectors erased*/
Uint32 SPI_EEPROM_erase(Uint32 SPI_CS, Uint32 start_address, 
	Uint32 byteCount, Uint32 address_width)
{
	Uint32 uiAddress, uiSectorCount= 0;
	Uint32 uiAlignedAddress;	/*address align to sector*/
	Uint32 uiAlignedByteCount;	/*byte count be multiple of sector size*/

	if(!needEraseBeforeWrite)
		return 0;

	/*since SPI EEPROM is erased in sector, we need align the address 
	and byte count to sector size*/
	if(start_address&(uiSPI_EEPROM_sector_size-1))
	{/*not aligned*/
		uiAlignedAddress= start_address&(~(uiSPI_EEPROM_sector_size-1));
		byteCount+= (start_address&(uiSPI_EEPROM_sector_size-1));
	}
	else
		uiAlignedAddress= start_address;
	
	if(uiAlignedByteCount&(uiSPI_EEPROM_sector_size-1))
	{/*not aligned*/
		uiAlignedByteCount= (byteCount+uiSPI_EEPROM_sector_size)&
			(~(uiSPI_EEPROM_sector_size-1));
	}
	else
		uiAlignedByteCount= byteCount;
	
	for(uiAddress= uiAlignedAddress; 
		uiAddress< uiAlignedAddress+uiAlignedByteCount;
		uiAddress+= uiSPI_EEPROM_sector_size)
	{
		if(0==SPI_EEPROM_sector_erase(SPI_CS, uiAddress, address_width))
			return uiSectorCount;
		uiSectorCount++;
	}
	
	return uiSectorCount;
}

